#!/usr/bin/env ruby

#$:.unshift "../../lib"

require 'jabber4r/jabber4r'

(puts "bitmapper.rb 'account@host/resource' 'password'") & exit if ARGV.size < 2


require 'fox'

include Fox

class ImageWindow < FXMainWindow

  attr_reader :jabber_session
  
  include Responder

  def initialize(app)
    # Invoke base class initialize first
    super(app, "Jabber Image Sender (#{ARGV[0]}): - untitled", nil, nil, DECOR_ALL,
      0, 0, 850, 600, 0, 0)

    # Make some icons
    uplevelicon = getIcon("tbuplevel.png")

    # Status bar
    statusbar = FXStatusbar.new(self,
      LAYOUT_SIDE_BOTTOM|LAYOUT_FILL_X|STATUSBAR_WITH_DRAGCORNER)

    # Splitter
    splitter = FXSplitter.new(self, (LAYOUT_SIDE_TOP|LAYOUT_FILL_X|
      LAYOUT_FILL_Y| SPLITTER_TRACKING|SPLITTER_VERTICAL|SPLITTER_REVERSED))
  
    # Sunken border for image widget
    imagebox = FXHorizontalFrame.new(splitter,
      FRAME_SUNKEN|FRAME_THICK|LAYOUT_FILL_X|LAYOUT_FILL_Y,
      0, 0, 0, 0, 0, 0, 0, 0)
  
    # Make image widget
    @imageview = FXImageView.new(imagebox, nil, nil, 0,
      LAYOUT_FILL_X|LAYOUT_FILL_Y)
    
    # Sunken border for file list
    @filebox = FXHorizontalFrame.new(splitter, LAYOUT_FILL_X|LAYOUT_FILL_Y,
      0, 0, 0, 0, 0, 0, 0, 0)
  
    # Make file list
    fileframe = FXHorizontalFrame.new(@filebox,
      FRAME_SUNKEN|FRAME_THICK|LAYOUT_FILL_X|LAYOUT_FILL_Y,
      0, 0, 0, 0, 0, 0, 0, 0, 0, 0)
    @filelist = FXFileList.new(fileframe, nil, 0,
      LAYOUT_FILL_X|LAYOUT_FILL_Y|ICONLIST_MINI_ICONS|ICONLIST_AUTOSIZE)
    @filelist.connect(SEL_DOUBLECLICKED, method(:onCmdFileList))
    FXButton.new(@filebox, "\tUp one level\tGo up to higher directory.",
      uplevelicon, @filelist, FXFileList::ID_DIRECTORY_UP,
      BUTTON_TOOLBAR|FRAME_RAISED|LAYOUT_FILL_Y)
      
    # Initialize file name and pattern for file dialog
    @filename = "untitled"
    @preferredFileFilter = 0
  end

  # Convenience function to construct a PNG icon
  def getIcon(filename)
    FXPNGIcon.new(getApp(), File.open(filename, "rb").read)
  end

  def hasExtension(filename, ext)
    File.basename(filename, ext) != File.basename(filename)
  end
  
  def loadImage(file)
    sendJabberImage(file)
    updateImage(file)
  end

  # Load the named image file
  def updateImage(file)
    img = nil
    if hasExtension(file, ".gif")
      img = FXGIFImage.new(getApp(), nil, IMAGE_KEEP|IMAGE_SHMI|IMAGE_SHMP)
    elsif hasExtension(file, ".bmp")
      img = FXBMPImage.new(getApp(), nil, IMAGE_KEEP|IMAGE_SHMI|IMAGE_SHMP)
    elsif hasExtension(file, ".xpm")
      img = FXXPMImage.new(getApp(), nil, IMAGE_KEEP|IMAGE_SHMI|IMAGE_SHMP)
    elsif hasExtension(file, ".png")
      img = FXPNGImage.new(getApp(), nil, IMAGE_KEEP|IMAGE_SHMI|IMAGE_SHMP)
    elsif hasExtension(file, ".jpg")
      img = FXJPGImage.new(getApp(), nil, IMAGE_KEEP|IMAGE_SHMI|IMAGE_SHMP)
    elsif hasExtension(file, ".pcx")
      img = FXPCXImage.new(getApp(), nil, IMAGE_KEEP|IMAGE_SHMI|IMAGE_SHMP)
    elsif hasExtension(file, ".tif")
      img = FXTIFImage.new(getApp(), nil, IMAGE_KEEP|IMAGE_SHMI|IMAGE_SHMP)
    elsif hasExtension(file, ".tga")
      img = FXTGAImage.new(getApp(), nil, IMAGE_KEEP|IMAGE_SHMI|IMAGE_SHMP)
    elsif hasExtension(file, ".ico")
      img = FXICOImage.new(getApp(), nil, IMAGE_KEEP|IMAGE_SHMI|IMAGE_SHMP)
    end

    # Perhaps failed?
    if !img
      FXMessageBox.error(self, MBOX_OK, "Error loading image",
        "Unsupported image type: #{file}")
      return
    end

    # Load it...
    getApp().beginWaitCursor do
      FXFileStream.open(file, FXStreamLoad) { |stream| img.loadPixels(stream) }
      img.create
      @imageview.image = img
    end
  end

  # Quit the application
  def onCmdQuit(sender, sel, ptr)
    getApp().exit(0)
  end

  # Command message from the file list
  def onCmdFileList(sender, sel, index)
    if index >= 0
      if @filelist.isItemDirectory(index)
        @filelist.directory = @filelist.getItemPathname(index)
      elsif @filelist.isItemFile(index)
        @filename = @filelist.getItemPathname(index)
        loadImage(@filename)
      end
    end
    return 1
  end


  # Create and show window
  def create
    dir = "."
    @filebox.height = 100
    super   # i.e. FXMainWindow::create()
    show(PLACEMENT_SCREEN)
  end
  
  def connectToJabber(account, account_password)
    begin
      @jabber_session = Jabber::Session.bind_digest("#{account}", "#{account_password}")
      @jabber_session.add_message_listener do |message|
        File.open("TMP"+message.subject, "wb") do |file|
          file.write message.body.unpack("m")[0]
          file.flush
        end
        updateImage("TMP"+message.subject)
        File.delete("TMP"+message.subject)
      end
    rescue
      puts "Account does not exist: #{account} - #{account_password}"
      puts $!
      puts $!.backtrace
      exit
    end
  end
  
  def sendJabberImage(imagefile)
    @jabber_session.roster.each_item do |item|
      item.each_resource do |resource|
        if resource.name == "bitmapper"
          data = nil
          File.open(imagefile, "rb") {|file| data = file.read}
          data = [data].pack("m")
          @jabber_session.new_chat_message(item.jid.to_s+"/bitmapper").set_subject(File.basename(imagefile)).set_body(data).send
        end
      end
    end
  end
  
  def disconnectFromJabber
    @jabber_session.close if @jabber_session
  end
end

if $0==__FILE__
  # Make application
  application = FXApp.new("ImageViewer", "FoxTest")
  # Make window
  window = ImageWindow.new(application)

  # Handle interrupts to terminate program gracefully
  application.addSignal("SIGINT", window.method(:onCmdQuit))

  # Create it
  application.create
  
  # Connect to Jabber
  window.connectToJabber(*ARGV)

  # Run
  application.run
  
  # Shutdown Jabber connection
  window.disconnectFromJabber
  
end

